!
! Module: impoissonize_mod
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: This creates an image whose values are random integers, each obeying a Poissonian probability distribution about a mean given by the respective value of the input image.
!
! Comments: 
! 
! $Id: impoissonize_mod.f90,v 1.1 2013/09/06 12:00:44 eojero Exp $
!- 
module impoissonize_mod

! You should write the module name in the 'use' statement, the package name
! (and version number) in the DEPEND file, and the * from the lib*.a name in
! <package>/src/Makefile.

  use dal
  use cal
  use param  
!  use errorHandling

  use dal_aux
  use random_aux
  use save_image

  implicit none 

  private 

  ! Command-line parameters:
  character(255) ::&
    inImageSetName,&
    outImageSetName,&
    tempImageSetName

  ! Other globals:

  public impoissonize

contains 

  !---------------------------------------------------------------------
  subroutine impoissonize
  !---------------------------------------------------------------------

    implicit none

    real(single), pointer :: imageReal(:,:)
    integer(int32), allocatable :: imageInt(:,:)

    !-------------------------------------------------------------------
    ! Start

    call readParameters

    call readArrayData(inImageSetName, imageReal) ! in ssclib/dal_aux
    allocate(imageInt(size(imageReal,1),size(imageReal,2)))
    call eventizeImage(imageReal, imageInt)
    call saveImage(imageInt, inImageSetName, outImageSetName, tempImageSetName)
    ! in ssclib/save_image

    deallocate(imageReal)

  end subroutine impoissonize
  
  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    inImageSetName  = stringParameter('inset')
    outImageSetName = stringParameter('outset')
    tempImageSetName = stringParameter('tempset')

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine eventizeImage(imageReal, imageInt)
  !---------------------------------------------------------------------

    implicit none

    real(single),   intent(in)  :: imageReal(:,:)
    integer(int32), intent(out) :: imageInt(size(imageReal,1),size(imageReal,2))

    integer :: xi, yi

    !-------------------------------------------------------------------
    ! Start

    do xi = 1, size(imageReal, 1)
      do yi = 1, size(imageReal, 2)
        imageInt(xi, yi) = poissonRandom(imageReal(xi, yi))
        ! in ssclib/random_aux
      end do
    end do

  end subroutine eventizeImage

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module impoissonize_mod
