//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//                     (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	whatatt.cc: attitude inquiry task
//
//	DESCRIPTION:
//		A simple task to inquire the S/C attitude at a specified point in
//		time through the OAL_getAttitude service.
//
//	CHANGE HISTORY:
//		UL, 1999-03-19: 1.0.0: - initial version
//		UL, 2000-09-13: 1.1.0: - added 'bstoolsout' switch
//
//	AUTHOR:
//		U. Lammers, Fri Mar 19 11:00:41 MET 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/tools/src/whatatt/whatatt.cc,v 1.9 2013/09/05 14:51:50 eojero Exp $


#include "OalServer.h"

#include <iostream>
#include "errstr.h"
#include "param.h"
#include "STime.h"
#include "Celestial.h"
#include "whatatt.h"

void whatatt()
{
	const string reqtime(stringParameter("t"));
	const AttitudeLocator *attloc = oalServer.attitudeLocator();
	SpacecraftAttitude att;
	try {
		att = attloc->attitude(Xmm::toTimeTag(reqtime));
	} catch(...) {
		errstr<<"No attitude data available at t="<<Xmm::toTimeTag(reqtime)
			  <<Sas::error("NoData");
	}
	cout.precision(8);
	if (!booleanParameter("bstoolsout"))
		cout<<"Attitude at t="<<reqtime<<": "<<att<<endl;
	else
		cout<<"ra="<<att.ra()<<" dec="<<att.dec()<<" apos="<<att.apos();
}
