/*
    xmmtimeconv.cc

    Small utility task that given an input time outputs the equivalent
    in all known formats.

    (c) 2013 European Space Agency - written by Richard Saxton
*/

#include <param.h>		// SAS
#include <STime.h>

#include <iostream>		// C++

#include "Xmm.h"
#include "common.h"
#include "xmmtimeconv.h"

void xmmtimeconv()
{

   // Find the input string format
	string time_format=stringParameter("format");
    string time=stringParameter("time");

   // Try and read the input time
	STime t1;
	STime::Representation r = STime::MJD;
	STime t0(Xmm::missionReferenceTime);

    STime::errorHandling = STime::EXCEPTION;
    try {

       // Try a string representation
    	t1 = STime(time);

    } catch (STime::InvalidTime &iv) {

    	cout<<"Converting using the input format "<<time_format<<endl;

       // Convert time string into a double
		std::istringstream istr(time);
		double dtime=0.0;
		istr >> dtime;

       // Try a numerical representation
    	if (time_format == "MRT") {

    	  // Convert the number of seconds into an MJD
    		LeapSeconds *lss = new LeapSeconds;
    		short leaps = lss->howManyBetween(Xmm::missionReferenceTime.modifiedJulianDay().full(), dtime);
    		dtime = Xmm::missionReferenceTime.modifiedJulianDay().full() + (dtime / STime::secondsPerDay)-(leaps/STime::secondsPerDay);
    		r = STime::MJD;
    	}
    	else if (time_format== "MJD")
    		r = STime::MJD;
    	else if (time_format == "JD")
		    r = STime::JD;
		else if (time_format == "DYN")
	        r = STime::DYN;

        // Check for a valid decimal time
      	if (! STime::isValidTime(dtime, r))
      		errstr<<"Invalid input time "<<time
      		      <<Sas::error("InvalidInputTime");

      	// Should be ok
    	try {
    		t1 = STime(dtime, r);
    	} catch (STime::InvalidTime &iv) {
    		iv.error();
    	}
    }

    // Check time is after earliest time supported
 	if (t1 < STime::earliest)
  		errstr<<"Input time "<<time<<" must be after "<<STime::earliest.toString(3, STime::FITS)
  		      <<Sas::error("InvalidInputTime");

       // Show result in many formats
        cout<<"FITS time: "<<t1.toString(3, STime::FITS)<<endl;

		//cout<<setprecision(15)<<"Fractional year: "<<t1.decimalYearNumber()<<endl;

        cout<<setprecision(15)<<"Julian day: "<<t1.julianDay().full()<<endl;
        cout<<setprecision(15)<<"MJD: "<<t1.modifiedJulianDay().full()<<endl;
        cout<<setprecision(15)<<"Mission ref. time (MRT; secs since "<<t0.toString(3, STime::FITS)<<"): "
            <<(t1-Xmm::missionReferenceTime)<<endl;

        cout<<"Calendar: "<<t1.toString(0, STime::CALENDAR)<<endl;
}
